"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const react_1 = __importDefault(require("react"));
const Button_1 = require("../Button/Button");
const testing_library_1 = require("../__test__/testing-library");
const ForwardRefsDemo_1 = __importDefault(require("../storybook/utils/ForwardRefsDemo"));
const theme_1 = require("../theme/theme");
const createTheme_1 = require("../utils/createTheme");
const utils_1 = require("../utils/utils");
const Input_1 = require("./Input");
const id = 'fake-test-id';
describe('Input', () => {
    test('it renders', () => {
        const json = rendererCreateWithTheme(react_1.default.createElement(Input_1.Input, { id: id })).toJSON();
        expect(json).toMatchSnapshot();
    });
    test('it renders an input element with a red border with a color prop is passed', () => {
        const json = rendererCreateWithTheme(react_1.default.createElement(Input_1.Input, { id: id, color: 'red' })).toJSON();
        expect(json).toMatchSnapshot();
    });
    test('it renders an input element with a really large padding and margin', () => {
        const json = rendererCreateWithTheme(react_1.default.createElement(Input_1.Input, { id: id, p: 4, m: 4 })).toJSON();
        expect(json).toMatchSnapshot();
    });
    test('it renders an input element with large text', () => {
        const json = rendererCreateWithTheme(react_1.default.createElement(Input_1.Input, { id: id, fontSize: 4 })).toJSON();
        expect(json).toMatchSnapshot();
    });
    test('forwards ref to the input', () => {
        const target = 'TARGET';
        const placeholder = 'Apples';
        function refChild(dsRef) {
            function onClick() {
                dsRef.current.focus();
            }
            return (react_1.default.createElement(react_1.default.Fragment, null,
                react_1.default.createElement(Input_1.Input, { id: id, ref: dsRef, placeholder: placeholder }),
                react_1.default.createElement(Button_1.Button, { onClick: onClick, mt: 3 }, target)));
        }
        const { getByText, getByPlaceholderText } = (0, testing_library_1.render)(react_1.default.createElement(ForwardRefsDemo_1.default, { refChild: refChild }));
        testing_library_1.fireEvent.click(getByText(target));
        expect(getByPlaceholderText(placeholder)).toHaveFocus();
    });
    test('it renders disabled', () => {
        const paletteTheme = (0, createTheme_1.createTheme)(theme_1.theme);
        (0, testing_library_1.render)(react_1.default.createElement(Input_1.Input, { id: id, disabled: true, placeholder: 'Disabled Input' }));
        const input = testing_library_1.screen.getByPlaceholderText('Disabled Input');
        expect(input).toBeDisabled();
        expect(input).toHaveStyleRule('background-color', (0, utils_1.getPaletteColor)('background.light')({ theme: paletteTheme }), { modifier: ':disabled' });
        expect(input).toHaveStyleRule('color', (0, utils_1.getPaletteColor)('text.light')({ theme: paletteTheme }), {
            modifier: ':disabled',
        });
        expect(input).toHaveStyleRule('cursor', 'not-allowed', { modifier: ':disabled' });
    });
    describe('helper text', function () {
        test('renders with same color as Input', () => {
            const helperText = 'hello world';
            const color = 'error.base';
            const paletteTheme = (0, createTheme_1.createTheme)(theme_1.theme);
            const { getByText } = (0, testing_library_1.render)(react_1.default.createElement(Input_1.Input, { id: id, color: color, helperText: react_1.default.createElement(Input_1.Input.HelperText, null, helperText) }));
            const helperTextNode = getByText(helperText);
            expect(helperTextNode).not.toBeNull();
            expect(helperTextNode).toHaveStyleRule('color', (0, utils_1.getPaletteColor)(color)({ theme: paletteTheme }));
        });
        test('can override the color from Input', () => {
            const helperText = 'hello world';
            const placeholder = 'click me';
            const color = 'error.base';
            const helperTextColor = 'secondary.base';
            const paletteTheme = (0, createTheme_1.createTheme)(theme_1.theme);
            const { getByText, getByPlaceholderText } = (0, testing_library_1.render)(react_1.default.createElement(Input_1.Input, { id: id, color: color, placeholder: placeholder, helperText: react_1.default.createElement(Input_1.Input.HelperText, { color: helperTextColor }, helperText) }));
            const helperTextNode = getByText(helperText);
            expect(helperTextNode).not.toBeNull();
            expect(helperTextNode).toHaveStyleRule('color', (0, utils_1.getPaletteColor)(helperTextColor)({ theme: paletteTheme }));
            expect(getByPlaceholderText(placeholder)).toHaveStyleRule('border-color', (0, utils_1.getPaletteColor)(color)({ theme: paletteTheme }));
        });
    });
});
//# sourceMappingURL=Input.spec.js.map