"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const react_1 = __importDefault(require("react"));
const testing_library_1 = require("../__test__/testing-library");
const PasswordInput_1 = require("./PasswordInput");
const sampleRegexChecks = [
    { label: '1 Uppercase Letter', regex: /(?=.*[A-Z])/ },
    { label: '1 Lowercase Letter', regex: /(?=.*[a-z])/ },
];
describe('PasswordInput', () => {
    it('test show/hide password options', () => {
        (0, testing_library_1.render)(react_1.default.createElement(PasswordInput_1.PasswordInput, { hasProgressBar: true }));
        const inputField = testing_library_1.screen.getByTestId('input-field');
        const visibilityButton = testing_library_1.screen.getByTitle('visibility-button');
        expect(inputField).toHaveAttribute('type', 'password');
        testing_library_1.fireEvent.click(visibilityButton);
        expect(inputField).toHaveAttribute('type', 'text');
    });
    it('test with a strong password', () => {
        const mockOnChange = jest.fn();
        (0, testing_library_1.render)(react_1.default.createElement(PasswordInput_1.PasswordInput, { hasProgressBar: true, onChange: mockOnChange }));
        const inputField = testing_library_1.screen.getByTestId('input-field');
        testing_library_1.fireEvent.change(inputField, { target: { value: 'GoodPassword1!' } });
        expect(mockOnChange).toHaveBeenLastCalledWith({
            isValid: true,
            value: 'GoodPassword1!',
        });
        const checkIconOn = testing_library_1.screen.queryAllByTestId('check-icon-on');
        const checkIconOff = testing_library_1.screen.queryAllByTestId('check-icon-off');
        expect(checkIconOn).toHaveLength(5);
        expect(checkIconOff).toHaveLength(0);
        expect(checkIconOn[0]).toHaveStyleRule('color', '#0a0');
        const checkMark = testing_library_1.screen.queryAllByTestId('check-mark-icon');
        expect(checkMark).toHaveLength(1);
    });
    it('test with a no number and special character password', () => {
        const mockOnChange = jest.fn();
        (0, testing_library_1.render)(react_1.default.createElement(PasswordInput_1.PasswordInput, { hasProgressBar: true, onChange: mockOnChange }));
        const inputField = testing_library_1.screen.getByTestId('input-field');
        testing_library_1.fireEvent.change(inputField, { target: { value: 'SamplePassword' } });
        expect(mockOnChange).toHaveBeenLastCalledWith({
            isValid: false,
            value: 'SamplePassword',
        });
        const checkIconOn = testing_library_1.screen.queryAllByTestId('check-icon-on');
        const checkIconOff = testing_library_1.screen.queryAllByTestId('check-icon-off');
        expect(checkIconOn).toHaveLength(3);
        expect(checkIconOff).toHaveLength(2);
        expect(checkIconOn[0]).toHaveStyleRule('color', '#0a0');
        expect(checkIconOff[0]).toHaveStyleRule('color', '#4f6f8f');
        const checkMark = testing_library_1.screen.queryAllByTestId('check-mark-icon');
        expect(checkMark).toHaveLength(0);
    });
    it('test with a weak password', () => {
        const mockOnChange = jest.fn();
        (0, testing_library_1.render)(react_1.default.createElement(PasswordInput_1.PasswordInput, { hasProgressBar: true, onChange: mockOnChange }));
        const inputField = testing_library_1.screen.getByTestId('input-field');
        testing_library_1.fireEvent.change(inputField, { target: { value: 'bad' } });
        expect(mockOnChange).toHaveBeenLastCalledWith({
            isValid: false,
            value: 'bad',
        });
        const checkIconOn = testing_library_1.screen.queryAllByTestId('check-icon-on');
        const checkIconOff = testing_library_1.screen.queryAllByTestId('check-icon-off');
        expect(checkIconOn).toHaveLength(1);
        expect(checkIconOff).toHaveLength(4);
        expect(checkIconOff[0]).toHaveStyleRule('color', '#4f6f8f');
        const checkMark = testing_library_1.screen.queryAllByTestId('check-mark-icon');
        expect(checkMark).toHaveLength(0);
    });
    it('test with a manual regex checks', () => {
        const mockOnChange = jest.fn();
        (0, testing_library_1.render)(react_1.default.createElement(PasswordInput_1.PasswordInput, { hasProgressBar: true, regexChecks: sampleRegexChecks, onChange: mockOnChange }));
        const inputField = testing_library_1.screen.getByTestId('input-field');
        testing_library_1.fireEvent.change(inputField, { target: { value: 'Password' } });
        expect(mockOnChange).toHaveBeenLastCalledWith({
            isValid: true,
            value: 'Password',
        });
        const checkIconOn = testing_library_1.screen.queryAllByTestId('check-icon-on');
        const checkIconOff = testing_library_1.screen.queryAllByTestId('check-icon-off');
        expect(checkIconOn).toHaveLength(2);
        expect(checkIconOff).toHaveLength(0);
        expect(checkIconOn[0]).toHaveStyleRule('color', '#0a0');
        const checkMark = testing_library_1.screen.queryAllByTestId('check-mark-icon');
        expect(checkMark).toHaveLength(1);
    });
    it('ignores regex checks and returns valid if no progress bar', () => {
        const mockOnChange = jest.fn();
        (0, testing_library_1.render)(react_1.default.createElement(PasswordInput_1.PasswordInput, { label: 'Password', onChange: mockOnChange }));
        const inputField = testing_library_1.screen.getByTestId('input-field');
        testing_library_1.fireEvent.change(inputField, { target: { value: 'test' } });
        expect(mockOnChange).toHaveBeenCalledWith({ isValid: true, value: 'test' });
    });
});
//# sourceMappingURL=PasswordInput.spec.js.map