"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.textWrap = exports.textTransform = exports.colorSchemeCustomForeground = exports.colorScheme = exports.borders = exports.color = exports.getByPalette = exports.getTextColorOn = exports.getLinkStylesOn = exports.getValidPaletteColor = exports.getPaletteColor = exports.applyVariations = exports.applySizes = exports.getBreakpointSize = exports.getContrastRatio = exports.getLuminance = exports.decomposeColor = exports.hexToRgb = exports.deprecatedColorValue = exports.hasPaletteColorShade = exports.hasPaletteColor = void 0;
const theme_get_1 = __importDefault(require("@styled-system/theme-get"));
const styled_components_1 = require("styled-components");
const theme_1 = require("../theme");
/**
 * Checks if the given color prop is a valid palette color
 * @public
 */
const hasPaletteColor = (props) => {
    return (props.theme &&
        props.theme.palette &&
        typeof props.color === 'string' &&
        Object.keys(props.theme.palette).includes(props.color.split('.')[0]));
};
exports.hasPaletteColor = hasPaletteColor;
/**
 * @public
 */
const hasPaletteColorShade = (props, color = undefined) => {
    const validColor = (typeof color === 'string' && color) || (typeof props.color === 'string' && props.color);
    const [col, shade] = validColor.split('.');
    return (props.theme &&
        props.theme.palette &&
        Object.keys(props.theme.palette).includes(col) &&
        Object.keys(props.theme.palette[col]).includes(shade));
};
exports.hasPaletteColorShade = hasPaletteColorShade;
/**
 * @public
 */
const deprecatedColorValue = () => (props, propName, componentName) => {
    if (process.env.NODE_ENV !== 'production' &&
        props.theme &&
        props[propName] &&
        !(0, exports.hasPaletteColor)({ color: props[propName], ...props })) {
        return new Error(`The color value of \`${props[propName]}\` for \`${componentName}\` is deprecated and will be removed in a future release. Please use a palette color instead.`);
    }
};
exports.deprecatedColorValue = deprecatedColorValue;
/**
 * Converts a hex color to rgb
 *
 * @example hexToRgb('#0068EF') =\> 'rgb(0, 104, 239)'
 *
 * @param color - The color to transform to rgb
 *
 * @returns The color in rgb
 *
 * @public
 */
const hexToRgb = (color) => {
    color = color.substring(1);
    /* eslint-disable @rushstack/security/no-unsafe-regexp */
    let colors = color.match(new RegExp(`.{1,${color.length / 3}}`, 'g'));
    if (colors) {
        colors = colors.map((val) => parseInt(val.length === 1 ? val + val : val, 16)).join(', ');
    }
    return colors ? `rgb(${colors})` : '';
};
exports.hexToRgb = hexToRgb;
/**
 * Decomposes a color into an array of values
 *
 * @example decomposeColor('#0068EF') =\> [0, 122, 255]
 *
 * @param color - The color to decompose
 *
 * @returns An array of the color values
 *
 * @public
 */
const decomposeColor = (color) => {
    if (color.charAt(0) === '#') {
        return (0, exports.decomposeColor)((0, exports.hexToRgb)(color));
    }
    return color
        .substring(color.indexOf('(') + 1, color.length - 1)
        .split(',')
        .map((value) => parseFloat(value));
};
exports.decomposeColor = decomposeColor;
/**
 * Gets the luminance of a color
 *
 * @example getLuminance('#0068EF') =\> 0.211
 * @see https://www.w3.org/TR/WCAG20-TECHS/G17.html#G17-tests
 *
 * @param color - The color to get the luminance of
 *
 * @returns The luminance of the color
 *
 * @public
 */
const getLuminance = (color) => {
    const [r, g, b] = (0, exports.decomposeColor)(color).map((val) => {
        val = val / 255;
        return val <= 0.03928 ? val / 12.92 : ((val + 0.055) / 1.055) ** 2.4;
    });
    return 0.2126 * r + 0.7152 * g + 0.0722 * b;
};
exports.getLuminance = getLuminance;
/**
 * Gets the contrast ratio between two colors
 *
 * @example getContrastRatio('#0068EF', '#fff') =\> 4.016975780478911
 * @see https://www.w3.org/TR/WCAG20-TECHS/G17.html#G17-tests
 *
 * @public
 */
const getContrastRatio = (colorA, colorB) => {
    const luminA = (0, exports.getLuminance)(colorA);
    const luminB = (0, exports.getLuminance)(colorB);
    return (Math.max(luminA, luminB) + 0.05) / (Math.min(luminA, luminB) + 0.05);
};
exports.getContrastRatio = getContrastRatio;
/**
 *
 * @param array - An array of sizes, ex: ['small', null, null, null, null, 'medium']
 * @param length - A number: this will be used to slice the above array
 *
 * @returns This function's purpose is to determine which item in the provided array to return at each breakpoint
 * 1) takes in an array, splits the array at the given length and creates a new smaller array
 * 2) that new smaller array is then passed into a reduceRight function to determine which value to be returned
 *  reduceRight is used so that the reduce function reads from right to left
 *
 * @public
 */
const getBreakpointSize = (array, length) => {
    const filteredArray = array.slice(0, length);
    return filteredArray.reduceRight((carry, item) => {
        return !carry ? item : carry;
    }, null);
};
exports.getBreakpointSize = getBreakpointSize;
/**
 * Applies the selected size styles to a styled component.
 * if size is a string, it will pass that string into the sizes object to return the correct css
 * if size is an array, it will determine the current screen size and then utilize the getBreakpointSize
 * helper to determine the css to return from the sizes object
 *
 * @param sizes - An object of size styles
 *
 * @public
 */
const applySizes = (sizes = null, defaultSize = 'medium', mediaQueriesOptions = theme_1.mediaQueries) => ({ size }) => {
    if (sizes && typeof size === 'string') {
        // prettier-ignore
        return (0, styled_components_1.css) `${sizes[size] || sizes[defaultSize] || ''}`;
    }
    if (sizes && Array.isArray(size)) {
        // prettier-ignore
        return (0, styled_components_1.css) `${sizes[(0, exports.getBreakpointSize)(size, 1)]};
        ${mediaQueriesOptions.sm} {
          ${sizes[(0, exports.getBreakpointSize)(size, 2)]};
        }
        ${mediaQueriesOptions.md} {
          ${sizes[(0, exports.getBreakpointSize)(size, 3)]};
        }
        ${mediaQueriesOptions.lg} {
          ${sizes[(0, exports.getBreakpointSize)(size, 4)]};
        }
        ${mediaQueriesOptions.xl} {
          ${sizes[(0, exports.getBreakpointSize)(size, 5)]};
        }
        ${mediaQueriesOptions.xxl} {
          ${sizes[(0, exports.getBreakpointSize)(size, 6)]};
        }`;
    }
};
exports.applySizes = applySizes;
const colorShadeRegex = /^([a-z]+)\.([a-z]+)$/i;
/**
 * Applies the selected variation style to a styled component.
 * Combines the variation style with any custom styling from
 * theme.componentStyles[component][variation]
 *
 * Once updated to styled-components v4, componentName is no
 * longer needed as it is part of forwardedClass.displayName
 *
 * @param componentName - The name of the component
 * @param variations - An object of variation styles
 *
 * @public
 */
const applyVariations = (componentName, variations = null) => (props) => {
    let { color } = props;
    const { variation } = props;
    const colorShade = !!color && typeof color === 'string' && color.match(colorShadeRegex);
    let shade;
    if (colorShade) {
        color = colorShade[1];
        shade = colorShade[2];
    }
    const isValidShade = shade && typeof shade === 'string';
    if (variations && typeof variation === 'string') {
        if (isValidShade) {
            // prettier-ignore
            return (0, styled_components_1.css) `${variations[variation] || ''}
          ${typeof color === 'string' &&
                (0, theme_get_1.default)(`componentStyles.${componentName}.${variation}.${color}.${shade}`, '')}`;
        }
        // prettier-ignore
        return (0, styled_components_1.css) `${variations[variation] || ''}
        ${typeof color === 'string' && (0, theme_get_1.default)(`componentStyles.${componentName}.${variation}.${color}`, '')}`;
    }
    if (isValidShade) {
        // prettier-ignore
        return (0, styled_components_1.css) `${(0, theme_get_1.default)(`componentStyles.${componentName}.${color}.${shade}`, '')}`;
    }
    // prettier-ignore
    return (0, styled_components_1.css) `${(0, theme_get_1.default)(`componentStyles.${componentName}.${color}`, '')}`;
};
exports.applyVariations = applyVariations;
/**
 * Gets the color of a palette shade, using props.color as
 * the palette color. If palette shade does not exist, falls
 * back to theme.colors
 *
 * @example getPaletteColor('dark')(props) =\> will return the dark
 * shade of theme.palette[props.color].dark
 * @example getPaletteColor('primary.base')(props) =\> theme.palette.primary.base
 * @example getPaletteColor('primary', 'base')(props) =\> theme.palette.primary.base
 *
 * @public
 */
const getPaletteColor = (...args) => (props) => {
    let color = args.length === 2 ? args[0] : props.color;
    let shade = args.length === 2 ? args[1] : args[0];
    const colorShade = shade.match(colorShadeRegex);
    if (colorShade) {
        color = colorShade[0];
        shade = colorShade[1];
    }
    return ((0, theme_get_1.default)(`palette.${color}.${shade}`)(props) ||
        (0, theme_get_1.default)(`palette.${color}`)(props) ||
        (0, theme_get_1.default)(`colors.${color}`)(props) ||
        color);
};
exports.getPaletteColor = getPaletteColor;
/**
 * @public
 */
const getValidPaletteColor = (color) => (props) => {
    if (!color || typeof color !== 'string') {
        return null;
    }
    const [col, shade] = color.split('.');
    const newColor = `${col}.${shade || 'base'}`;
    if ((0, exports.hasPaletteColorShade)(props, newColor)) {
        return (0, exports.getPaletteColor)(newColor)(props);
    }
    return null;
};
exports.getValidPaletteColor = getValidPaletteColor;
/**
 * Gets css styles that make links more accessible depending on a given background color
 *
 * @param name       - The name of the background color
 * @param lightColor - Optional light color to use if it meets the contrast ratio, default is text.lightest
 * @param darkColor  - Optional dark color to use if it meets the contrast ratio, default is text.base
 * @param isBold     - Determines the font weight if an alternative color for the link is picked
 *
 * @public
 */
const getLinkStylesOn = (name, lightColor = 'text.lightest', darkColor = 'text.base', isBold = false) => (props) => {
    const { theme } = props;
    if (theme.palette) {
        const backgroundColor = (0, exports.getPaletteColor)(name)(props);
        const linkBaseColor = theme.palette.primary.base;
        const linkHoverColor = theme.palette.primary.dark;
        lightColor = (0, exports.getValidPaletteColor)(lightColor)(props);
        darkColor = (0, exports.getValidPaletteColor)(darkColor)(props);
        if (backgroundColor) {
            const hasDefaultContrast = (0, exports.getContrastRatio)(backgroundColor, linkBaseColor) >= theme.contrastRatio &&
                (0, exports.getContrastRatio)(backgroundColor, linkHoverColor) >= theme.contrastRatio;
            const hasLightContrast = (0, exports.getContrastRatio)(backgroundColor, lightColor) >= theme.contrastRatio;
            if (!hasDefaultContrast) {
                const fontWeight = isBold ? 'bold' : 'inherit';
                const contrastLinkColor = hasLightContrast ? lightColor : darkColor;
                // prettier-ignore
                return (0, styled_components_1.css) `color: ${contrastLinkColor}; font-weight: ${fontWeight}; text-decoration: underline; :hover { color: ${contrastLinkColor}; }`;
            }
        }
    }
    return '';
};
exports.getLinkStylesOn = getLinkStylesOn;
/**
 * Gets the text color that belongs on a given background color
 *
 * @param name - The name of the background color
 * @public
 */
const getTextColorOn = (name, lightColor = null, darkColor = null) => (props) => {
    const { theme } = props;
    if (theme.palette) {
        const color = (0, exports.getPaletteColor)(name)(props);
        const text = theme.palette.text;
        lightColor = (0, exports.getValidPaletteColor)(lightColor)(props) || text.lightest;
        darkColor = (0, exports.getValidPaletteColor)(darkColor)(props) || text.base;
        if (color) {
            return (0, exports.getContrastRatio)(lightColor, color) >= theme.contrastRatio ? lightColor : darkColor;
        }
        return darkColor;
    }
    return '';
};
exports.getTextColorOn = getTextColorOn;
/**
 * @public
 */
// prettier-ignore
const getByPalette = (props) => (0, styled_components_1.css) `background-color: ${(0, exports.getPaletteColor)(props.bg, 'base')(props)};color: ${(0, exports.getPaletteColor)(props.color, 'base')(props)};`;
exports.getByPalette = getByPalette;
/**
 * @public
 * Extended color function from styled-system. First checks
 * for a palette color before falling back to styled-system
 */
const color = (props) => {
    if (!props.theme || (!props.color && !props.bg)) {
        return '';
    }
    else if (props.color === 'text') {
        return props.color && props.bg
            ? (0, exports.getByPalette)(props)
            : // prettier-ignore
                (0, styled_components_1.css) `color: ${(0, exports.getPaletteColor)('base')(props)};`;
    }
    else if (props.color && props.bg) {
        return (0, exports.getByPalette)(props);
    }
    else if (props.color && (0, exports.hasPaletteColor)(props)) {
        // prettier-ignore
        return (0, styled_components_1.css) `background-color: ${(0, exports.getPaletteColor)('base')(props)};
color: ${(0, exports.getTextColorOn)('base')(props)};`;
    }
    else if (props.color) {
        // prettier-ignore
        return (0, styled_components_1.css) `color: ${(0, exports.getPaletteColor)('base')(props)};`;
    }
    else {
        // prettier-ignore
        return (0, styled_components_1.css) `background-color: ${(0, exports.getPaletteColor)(props.bg, 'base')(props)};`;
    }
};
exports.color = color;
/**
 * @public
 */
const borders = (props) => {
    const borderColor = props.color ? (0, exports.getPaletteColor)('base')(props) : (0, exports.getPaletteColor)('border.base')(props);
    const focusColor = props.color ? borderColor : (0, exports.getPaletteColor)('primary.base')(props);
    return {
        'border-color': borderColor,
        ':focus': {
            outline: 0,
            'border-color': focusColor,
            'box-shadow': `0 0 0 2px ${focusColor}`,
        },
    };
};
exports.borders = borders;
/**
 * @public
 */
const colorScheme = ({ colorScheme, ...props }) => {
    if (!colorScheme)
        return '';
    const { foreground, background } = (0, theme_get_1.default)(`colorSchemes.${colorScheme}`)(props);
    return `
    background-color: ${background};
    border-color: ${foreground};
    color: ${foreground};
  `;
};
exports.colorScheme = colorScheme;
/**
 * @public
 * Use this for components where color should not be set based on the colorScheme's foreground.
 * If a color prop is provided, that color will be used for the foreground color. Otherwise,
 * the foreground color will be set automatically to text.lightest or text.base depending on the
 * colorScheme's background color.
 */
const colorSchemeCustomForeground = ({ colorScheme, color, iconUsesColorScheme, ...props }) => {
    if (!colorScheme)
        return '';
    const { foreground, background, backgroundName = '' } = (0, theme_get_1.default)(`colorSchemes.${colorScheme}`)(props);
    let paletteColor;
    if (color) {
        paletteColor = (0, exports.getPaletteColor)(color)(props);
    }
    return `
    background-color: ${background};
    border-color: ${foreground};
    color: ${paletteColor ? paletteColor : (0, exports.getTextColorOn)(backgroundName)(props)};

    ${iconUsesColorScheme &&
        `
        svg {
          color: ${foreground};
        }
      `}
  `;
};
exports.colorSchemeCustomForeground = colorSchemeCustomForeground;
/**
 * @public
 */
const textTransform = (props) => (props.textTransform ? { textTransform: props.textTransform } : null);
exports.textTransform = textTransform;
/**
 * @public
 */
const textWrap = (props) => (props.textWrap ? { textWrap: props.textWrap } : null);
exports.textWrap = textWrap;
//# sourceMappingURL=utils.js.map