/**
 * Checks if the given color prop is a valid palette color
 * @public
 */
export declare const hasPaletteColor: (props: any) => boolean;
/**
 * @public
 */
export declare const hasPaletteColorShade: (props: any, color?: any) => boolean;
/**
 * @public
 */
export declare const deprecatedColorValue: () => (props: any, propName: any, componentName: any) => Error;
/**
 * Converts a hex color to rgb
 *
 * @example hexToRgb('#0068EF') =\> 'rgb(0, 104, 239)'
 *
 * @param color - The color to transform to rgb
 *
 * @returns The color in rgb
 *
 * @public
 */
export declare const hexToRgb: (color: any) => string;
/**
 * Decomposes a color into an array of values
 *
 * @example decomposeColor('#0068EF') =\> [0, 122, 255]
 *
 * @param color - The color to decompose
 *
 * @returns An array of the color values
 *
 * @public
 */
export declare const decomposeColor: (color: any) => any;
/**
 * Gets the luminance of a color
 *
 * @example getLuminance('#0068EF') =\> 0.211
 * @see https://www.w3.org/TR/WCAG20-TECHS/G17.html#G17-tests
 *
 * @param color - The color to get the luminance of
 *
 * @returns The luminance of the color
 *
 * @public
 */
export declare const getLuminance: (color: any) => number;
/**
 * Gets the contrast ratio between two colors
 *
 * @example getContrastRatio('#0068EF', '#fff') =\> 4.016975780478911
 * @see https://www.w3.org/TR/WCAG20-TECHS/G17.html#G17-tests
 *
 * @public
 */
export declare const getContrastRatio: (colorA: any, colorB: any) => number;
/**
 *
 * @param array - An array of sizes, ex: ['small', null, null, null, null, 'medium']
 * @param length - A number: this will be used to slice the above array
 *
 * @returns This function's purpose is to determine which item in the provided array to return at each breakpoint
 * 1) takes in an array, splits the array at the given length and creates a new smaller array
 * 2) that new smaller array is then passed into a reduceRight function to determine which value to be returned
 *  reduceRight is used so that the reduce function reads from right to left
 *
 * @public
 */
export declare const getBreakpointSize: (array: any, length: any) => any;
/**
 * Applies the selected size styles to a styled component.
 * if size is a string, it will pass that string into the sizes object to return the correct css
 * if size is an array, it will determine the current screen size and then utilize the getBreakpointSize
 * helper to determine the css to return from the sizes object
 *
 * @param sizes - An object of size styles
 *
 * @public
 */
export declare const applySizes: (sizes?: any, defaultSize?: string, mediaQueriesOptions?: any) => ({ size }: {
    size: any;
}) => any;
/**
 * Applies the selected variation style to a styled component.
 * Combines the variation style with any custom styling from
 * theme.componentStyles[component][variation]
 *
 * Once updated to styled-components v4, componentName is no
 * longer needed as it is part of forwardedClass.displayName
 *
 * @param componentName - The name of the component
 * @param variations - An object of variation styles
 *
 * @public
 */
export declare const applyVariations: (componentName: any, variations?: any) => (props: any) => any;
/**
 * Gets the color of a palette shade, using props.color as
 * the palette color. If palette shade does not exist, falls
 * back to theme.colors
 *
 * @example getPaletteColor('dark')(props) =\> will return the dark
 * shade of theme.palette[props.color].dark
 * @example getPaletteColor('primary.base')(props) =\> theme.palette.primary.base
 * @example getPaletteColor('primary', 'base')(props) =\> theme.palette.primary.base
 *
 * @public
 */
export declare const getPaletteColor: (...args: any[]) => (props: any) => any;
/**
 * @public
 */
export declare const getValidPaletteColor: (color: any) => (props: any) => any;
/**
 * Gets css styles that make links more accessible depending on a given background color
 *
 * @param name       - The name of the background color
 * @param lightColor - Optional light color to use if it meets the contrast ratio, default is text.lightest
 * @param darkColor  - Optional dark color to use if it meets the contrast ratio, default is text.base
 * @param isBold     - Determines the font weight if an alternative color for the link is picked
 *
 * @public
 */
export declare const getLinkStylesOn: (name: any, lightColor?: string, darkColor?: string, isBold?: boolean) => (props: any) => any;
/**
 * Gets the text color that belongs on a given background color
 *
 * @param name - The name of the background color
 * @public
 */
export declare const getTextColorOn: (name: any, lightColor?: any, darkColor?: any) => (props: any) => any;
/**
 * @public
 */
export declare const getByPalette: (props: any) => any;
/**
 * @public
 * Extended color function from styled-system. First checks
 * for a palette color before falling back to styled-system
 */
export declare const color: (props: any) => any;
/**
 * @public
 */
export declare const borders: (props: any) => {
    'border-color': any;
    ':focus': {
        outline: number;
        'border-color': any;
        'box-shadow': string;
    };
};
/**
 * @public
 */
export declare const colorScheme: ({ colorScheme, ...props }: {
    [x: string]: any;
    colorScheme: any;
}) => string;
/**
 * @public
 * Use this for components where color should not be set based on the colorScheme's foreground.
 * If a color prop is provided, that color will be used for the foreground color. Otherwise,
 * the foreground color will be set automatically to text.lightest or text.base depending on the
 * colorScheme's background color.
 */
export declare const colorSchemeCustomForeground: ({ colorScheme, color, iconUsesColorScheme, ...props }: {
    [x: string]: any;
    colorScheme: any;
    color: any;
    iconUsesColorScheme: any;
}) => string;
/**
 * @public
 */
export declare const textTransform: (props: any) => {
    textTransform: any;
};
/**
 * @public
 */
export declare const textWrap: (props: any) => {
    textWrap: any;
};
//# sourceMappingURL=utils.d.ts.map